@@ -1,46 +1,39 @@
 package WWW::Yahoo::KeywordExtractor;
 
-use strict;
 use warnings;
+use strict;
 
+use Digest::MD5 qw(md5_hex);
 use LWP::UserAgent;
+use URI::Escape;
+use XML::Simple;
 
-use constant KEYWORD_API_URL => 'http://api.search.yahoo.com/ContentAnalysisService/V1/termExtraction';
-
-our $VERSION = '0.5.2';
+our $VERSION = '0.01';
 
 sub new {
 	my ($class, %args) = @_;
 	my $self = bless {%args}, $class;
+	$self->{'ua'} = LWP::UserAgent->new;
 	return $self;
 }
 
 sub extract {
-    my ($self, $content) = @_;
-    if (! $content) { die 'No content specified'; }
-    my $ua = LWP::UserAgent->new();
-    my $response = $ua->post(
-        KEYWORD_API_URL,
-        { 'appid' => 'WWWYahooKeywordExtractor',
-        'query' => 'null',
-        'context' => $content, }
-    );
-    if (! $response->is_success()) {
-    	die "Error getting data!\n";
-    }
-    my $xml = $response->content();
-    my @results = ();
-    while ($xml =~ m!<Result>([^<]*)</Result>!g) {
-        push @results, $1;
-    }
-    return \@results;
+	my ($self, %args) = @_;
+	if (! $args{'content'}) { die 'No content specified'; }
+	my $content_hash = md5_hex($args{'content'});
+	if (! $self->{'__cache'.$content_hash}) {
+		my $response = $self->{'ua'}->get('http://api.search.yahoo.com/ContentAnalysisService/V1/termExtraction?appid=WWWYahooKeywordExtractor&query=null&context='.uri_escape($args{'content'}));
+		if (! $response->is_success) {
+			die "Error getting data!\n";
+		}
+		my $xml = $response->content;
+		my $ref = XMLin($xml, ForceArray => [ 'Result' ]);
+		my @results = @{$ref->{'Result'}};
+		$self->{'__cache'.$content_hash} = \@results;
+	}
+	return $self->{'__cache'.$content_hash};
 }
 
-1;
-__END__
-
-=pod
-
 =head1 NAME
 
 WWW::Yahoo::KeywordExtractor - Get keywords from summary text via the Yahoo API
@@ -52,7 +45,7 @@ a list of relevant keywords.
 
   use WWW::Yahoo::KeywordExtractor;
   my $yke = WWW::Yahoo::KeywordExtractor->new();
-  my $keywords = $yke->extract('My wife and I love to cook together. Carolyn surprises me with new things to love about her everyday.');
+  my $keywords = $yke->extract(content => 'My wife and I love to cook together. Carolyn surprises me with new things to love about her everyday.');
   print join q{}. 'Keyword 1: ', $keywords->[0], "\n";
 
 =head1 SUBROUTINES/METHOD
@@ -66,8 +59,11 @@ The new subroutine creates and returns a WWW:Yahoo::KeywordExtractor object.
 This method will return a list of keywords based on sample data. It will die
 if there is no 'content' arg given.
 
-Note: In older versions this method would also cache the keywords returned,
-however this is no longer the case.
+=head1 CACHING
+
+This module will attempt to cache its data locally. It does this by creating
+content cache keys which are md5 hashes of content. Sooner or later I will
+update this module to also provide a list of content cache keys.
 
 =head1 AUTHOR
 
@@ -91,10 +87,6 @@ You can also look for information at:
 
 =over 4
 
-=item * WWW-Yahoo-KeywordExtractor SVN Repository
-
-L<http://code.sixapart.com/svn/WWW-Yahoo-KeywordExtractor>
-
 =item * AnnoCPAN: Annotated CPAN documentation
 
 L<http://annocpan.org/dist/WWW-Yahoo-KeywordExtractor>
@@ -117,9 +109,6 @@ L<http://search.cpan.org/dist/WWW-Yahoo-KeywordExtractor>
 
 Thanks to the bright developers at Yahoo for creating a nifty keyword API.
 
-Subbu Allamaraju ( http://www.subbu.org ) gave some good feedback and is also
-worth mentioning here.
-
 =head1 COPYRIGHT & LICENSE
 
 Copyright 2006 Nick Gerakines, all rights reserved.
@@ -128,3 +117,5 @@ This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
 
 =cut
+
+1;
